## VEHANT RESEARCH LAB challenge on “Smart City Surveillance: Unveiling Indian Person Attributes in Real Time”

## Usage
Our code is extended from https://github.com/Event-AHU/OpenPAR/tree/main/PromptPAR

### Requirements
we use a single V100 32GB GPU for training and evaluation.
```
conda create -n promptpar python=3.9
conda activate promptpar
pip install -r requirements.txt
```

### Env

Create a `.env` file in the root of this project with:

```shell
CUDA_VISIBLE_DEVICES=<devices to keep visible>
WANDB_API_KEY=<your weights and biases api key>
WANDB_ENTITY=<your entity>
WANDB_PROJECT=<your project>
cache_dir=/path/to/cache/.cache
data_dir=/path/to/folder/containing/VRL folder/data
images_dir=/path/to/folder/containing/VRL/folder/VRL/images
# vit checkpoint path
pretrain_path=/path/to/vit/checkpoint/jx_vit_base_p16_224-80ecf9dd.pth
```

### Dataset
We split the provided data into train (500) and val (100) splits using `dataset/preprocess/vrl_pad.py`. The test images are also re-numbered and added to the `images/` directory.

Data structure
```shell
data
|
|--VRL
    |
    |
    --images
```

### Checkpoint Download

Download the final finetuned [checkpoint](https://huggingface.co/cs-mshah/PromptPAR/tree/main) and save under the `checkpoints/` directory. The checkpoint would be `checkpoints/best_ma.pth`

## Training

Note: for disabling logging using weights and biases, append the shell command with `WANDB_MODE=disabled`.

We train on the PA100k dataset for 18 epochs using the `config.py` file as the config import in the `train.py` and other files where configs are imported.

```shell
python train.py PA100k --use_textprompt --use_div --use_vismask --use_GL --use_mm_former
```

We finetune further on the checkpoint from PA100k on the VRL dataset using the `vrl_config.py` file as the config import.
```shell
python train.py VRL --dir checkpoints/PA100k_epoch18.pth --checkpoint_attr 26 --use_div --use_vismask --use_GL --use_mm_former
```

## Test
For generating the `submission.txt`
```shell
python test.py VRL --valid_split test --checkpoint --dir checkpoints/best_ma.pth --use_div --use_vismask --use_GL --use_mm_former
```


## Experimental Results

The training logs can be found at [wandb](https://wandb.ai/manan-shah/promptpar). The test results are in the `submissions.txt` file.

