import autoroot
import os
import numpy as np
import random
import pickle

from easydict import EasyDict

np.random.seed(0)
random.seed(0)


attr_words = [
    "A pedestrian in black colored upper wear",
    "A pedestrian in blue colored upper wear",
    "A pedestrian in green colored upper wear",
    "A pedestrian in orange colored upper wear",
    "A pedestrian in red colored upper wear",
    "A pedestrian in white colored upper wear",
    "A pedestrian in yellow colored upper wear",
    "A pedestrian in multi colored upper wear",
    "A pedestrian in whose upper wear is not black and not blue and not green and not orange and not red and not white and not yellow and not multi colored",
# 
    "A pedestrian in black colored lower wear",
    "A pedestrian in blue colored lower wear",
    "A pedestrian in green colored lower wear",
    "A pedestrian in red colored lower wear",
    "A pedestrian in yellow colored lower wear",
    "A pedestrian in white colored lower wear",
    "A pedestrian in orange colored lower wear",
    "A pedestrian in whose lower wear is not black and not blue and not green and not orange and not red and not white and not yellow and not multi colored",
    "A pedestrian in multi colored lower wear",
# 
    "A pedestrian wearing a jacket as upper wear",
    "A pedestrian wearing a kurta as upper wear",
    "A pedestrian not wearning a jacket and not a kurta and not a saree and not a shirt and not a tshirt and not a woman suit and not a sweater",
    "A lady pedestrian wearing a saree as upper wear",
    "A pedestrian wearing a shirt as upper wear",
    "A lady pedestrian wearing a woman suite as upper wear",
    "A pedestrian wearing a tshirt as upper wear",
    "A pedestrian wearing a sweater as upper wear",
# 
    "A woman wearing leggings as lower wear",
    "A woman wearing a salwar as lower wear",
    "A pedestrian wearing shorts as lower wear",
    "A pedestrian wearing trousers as lower wear",
    "A pedestrian wearing jeans as lower wear",
    "A woman wearing a saree as lower wear",
    "A pedestrian not wearing leggings and not a salwar and not shorts and not trousers and not jeans and not a saree",
# 
    "A pedestrian in long-sleeved upper wear",
    "A pedestrian in short-sleeved upper wear",
    "A pedestrian in sleeveless upper wear",
# 
    "A pedestrian with a handbag",
    "A pedestrian with a backpack",
    "A pedestrian with no handbag and no backpack",
# 
    "A pedestrian with a headgear accessory",
# 
    "A pedestrian wearing sandals",
    "A pedestrian wearing shoes",
    "A pedestrian wearing slippers",
# 
    "A pedestrian in a sitting body posture",
    "A pedestrian in a lying body posture",
    "A pedestrian in a standing body posture",
# 
    "A pedestrian seen from the back",
    "A pedestrian seen from the front",
    "A pedestrian seen from the side",
]

neg_attr_words = [
]


def make_dir(path):
    if os.path.exists(path):
        pass
    else:
        os.mkdir(path)


def generate_data_description(save_dir):
    """
    create a dataset description file, which consists of images, labels
    """
    with open(os.path.join(save_dir, "train.txt"), "r") as train_file:

        vrl_data = np.loadtxt(train_file, dtype=int, delimiter=" ")

        dataset = EasyDict()
        dataset.description = "VRL"
        dataset.root = os.path.join(save_dir, "images")

        train_image_name = [f"{i+1}.jpg" for i in range(500)]
        val_image_name = [f"{i+1}.jpg" for i in range(500, 600)]
        test_image_name = [f"{i+1}.jpg" for i in range(600, 763)]
        dataset.image_name = train_image_name + val_image_name + test_image_name

        dataset.label = vrl_data[:, 1:]
        assert dataset.label.shape == (600, 49)

        test_label = np.zeros((163, 49), dtype=int)
        dataset.label = np.concatenate([dataset.label, test_label], axis=0)
        assert dataset.label.shape == (763, 49)

        dataset.attr_name = attr_words
        dataset.attributes = attr_words
        dataset.neg_attr_words = neg_attr_words
        dataset.expand_pos_attr_words = None
        dataset.expand_neg_attr_words = None

        dataset.partition = EasyDict()
        dataset.partition.train = np.arange(0, 500)
        dataset.partition.val = np.arange(500, 600)
        dataset.partition.test = np.arange(600, 763)
        dataset.partition.trainval = np.arange(0, 600)

        dataset.weight_train = np.mean(
            dataset.label[dataset.partition.train], axis=0
        ).astype(np.float32)
        dataset.weight_trainval = np.mean(
            dataset.label[dataset.partition.trainval], axis=0
        ).astype(np.float32)

        with open(os.path.join(save_dir, "pad.pkl"), "wb+") as f:
            pickle.dump(dataset, f)


if __name__ == "__main__":
    save_dir = os.path.join(os.environ["data_dir"], "VRL")
    generate_data_description(save_dir)
