import autoroot
import fiftyone as fo
from fiftyone.core.metadata import ImageMetadata
import os
import argparse


def main(args):
    if args.load_saved or fo.dataset_exists("VRL-PAR"):
        dataset = fo.load_dataset("VRL-PAR")
    else:
        dataset = fo.Dataset(name="VRL-PAR")
        train_size = 500
        val_size = 100
        for i, img in enumerate(os.listdir(args.images_dir)):
            img_path = os.path.join(args.images_dir, img)
            metadata = ImageMetadata.build_for(img_path)
            tags = []
            if i < train_size:
                tags.append("train")
            elif train_size <= i < train_size + val_size:
                tags.append("val")
            else:
                tags.append("test")
            sample = fo.Sample(filepath=img_path, metadata=metadata, tags=tags)
            dataset.add_sample(sample)

    session = fo.launch_app(dataset, remote=True)
    session.wait()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="View VRL PAR Challenge Dataset")
    parser.add_argument(
        "--images_dir",
        type=str,
        default=os.environ["images_dir"],
        help="Images directory",
    )
    parser.add_argument(
        "--load_saved",
        action="store_true",
        help="After creating the fo.Dataset once, \
          use this flag to load the already created dataset",
    )
    args = parser.parse_args()
    main(args)